{% extends "massmigration/base.html" %}

{% block content %}

<h1>Run Migration</h1>
<h2>{{migration.key}}</h2>
<p>{{migration.description}}</p>
<p class="pt">
	This will delete the <em>record</em> for the migration in the database.
</p>
<p>
	Once you have delete the record, you will be able to run the migration again.
</p>
<h3>If your migration might still be running...</h3>
<p>
	<strong>For simple migrations</strong>, deleting the migration record does not interrupt the processing of the migration on the backend.
	If you need to interrupt the processing you should stop the backend yourself.
</p>
<p>
	<strong>For mapper migrations</strong>, deleting the migration record will cause the migration
	to do a best effort to prevent any further operations being performed on the queryset,
	even if the backend is still trying to perform the processing.
	However, the prevention of operations is not guaranteed,
	so if you absolutely need to halt processing you should stop the backend yourself.
</p>
<p>
	For mapper migrations which have errored, the backend may still be trying to process the queryset.
	Again, the migration will try to prevent the operations being performed, but this is not guaranteed.
</p>

<form method="post" action="" class="pt">
	{% csrf_token %}
	<button type="submit">Delete migration record</button>
</form>

{% endblock %}
