{% extends "admin/change_list.html" %}


{% block result_list %}

	<div>
		<p>
			<a href="{% url 'massmigration_manage' %}" class="button">Manage Migrations</a>
		</p>
	</div>

	{{block.super}}
{% endblock %}
